::cisco::eem::event_register_timer watchdog name qos_timer sec 900 maxrun_sec 120

#------------------------------------------------------------------
# EEM policy to check qos drop packets from snmp agent.
#
# March 2007 - Christine Yan (cyan@cisco.com)
#
# Copyright (c) 2007-2008 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-04/25/2007 
# Title: report qos drop counters to customer
# Name: qos_drop_packet.tcl
# Desc: This script is an application to support template based qos drop \
#       counters.
#       
#       The script runs with an input-file and report drop counters based on
#       the input from tremplate-file.
#
#       The structure and requirment of the file is defined below.
# 
# Input-file:
#
#       The input file will tell which output customer wants to have. 
#       Each line would be a set of quardruple of the form. 
#       the format of this file would be:
#       diff_only counter, class name, interface name, input
#
#       The example of the input file can be as the following:
#       yes PingTest e0/0 input
#       (In above example, PingTest is the policymap name in the interface 
#        e0/0). 
#       no PingPackets gi0/0 output
#       (In above example, PingPackets is the classmap name in the interface
#        e0/0).
#
# Environment Variables for this script are:
#  _send_email_for_qos  (opt)
#  _svcdiag_qosConfigHistory (opt)
#  _svcdiag_qosEventHistory  (opt)
#  _qos_file_path  (mandatory)
#  _qos_input_file  (mandatory) 
#  _email_server (opt)
#  _email_from  (opt)
#  _email_to    (opt)
#  _email_cc    (opt)
#
# IOS config:
#
#  conf t
#
#  event manager environment _send_email_for_qos no
#  event manager environment _svcdiag_qosConfigHistory FALSE
#  event manager environment _svcdiag_qosEventHistory FALSE
#  event manager environment _qos_file_path disk0:/
#  event manager environment _qos_input_file inputfile
#  event manager directory user policy disk0:/svc_diag
#  event manager directory user library disk0:/user_lib
#  event manager policy qos_drop_packet.tcl type user
#
#  event manager environment _email_server email.cisco.com
#  event manager environment _email_from cyan@cisco.com
#  event manager environment _email_to cyan@cisco.com
#  event manager environment _email_cc cyan@cisco.com
#
#  Please note: on some devices you do have to use
#  event manager directory user policy disk2:/svc_diag
#  event manager directory user library disk2:/user_lib 
#
# For 3825 or 3xxx serial of routers, should use flash: instead of disk0:
#
# This script also needs a startard email format file to be present in 
# customer disk. The file name would be email_template_cmd.
# it should be present in :/userlib. You need to copy the standard email
# template from tftp server to xxx(disk_name):/user_lib
#
# When enable verbose for debugging purpose, "logging console gurantee" and 
# "logging buffer xxx(big_size)"  need to be configured for excessive messages 
# printed on screen.
#
#------------------------------------------------------------------
# The namespace import commands that follow import some cisco TCL extensions
# that are required to run this script

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*


# Check that envionment variable has been set

if {![info exists _qos_input_file]} {
    set result \
   "EEM policy Error: variable _qos_input_file has not been set"
    error $result "_qos_input_file is not set"
}

if {![info exists _qos_file_path]} {
    set result \
    "EEM policy Error: variable _qos_file_path has not been set"
    error $result "_qos_file_path is not set"
}

# User will make decision on if the email will be sent to them. This is 
# by environment variable _send_email_for_qos

set send_email 0

if {[info exists _send_email_for_qos]} {

    if [regexp yes $_send_email_for_qos] {

	# Check if all the env variables we need exist
	# If any of them doesn't exist, print out an error msg and quit

	if {![info exists _email_server]} {
	    set result \
		"Policy cannot be run: variable _email_server has not been set"
	    error $result "_email_server is required for sending email"
	}
	if {![info exists _email_from]} {
	    set result \
		"Policy cannot be run: variable _email_from has not been set"
	    error $result "_email_from is required for sending email"
	}
	if {![info exists _email_to]} {
	    set result \
		"Policy cannot be run: variable _email_to has not been set"
	    error $result "_email_to is required for sending email"
	}
	if {![info exists _email_cc]} {
	    #_email_cc is an option, must set to empty string if not set.
	    set _email_cc ""
	}

	set send_email 1	
    }
}     
   
global verbose
set verbose 0

if {[info exists _verbose_for_qos]} {

    if [regexp yes $_verbose_for_qos] {

	set verbose 1

    }
}

set config_history 0

if {[info exists _svcdiag_qosConfigHistory]} {
    if { $_svcdiag_qosConfigHistory == "TRUE" } {
	set config_history 1
    }
}

set event_history 0

if {[info exists _svcdiag_qosEventHistory]} {
    if { $_svcdiag_qosEventHistory == "TRUE" } {
	set event_history 1
    }
}

###############################################################################
#
#   Description: Get snmp whole oid from all kinds of index (one index and two
#                indexes) table queris.
#
#                This is a utility function to get snmp whole oid from one 
#                index or two index table queries. 
#                This function is not currently used. 
#
#   IN: snmp raw query
#   OUT: snmp oid with whole oid including static and index.
#
###############################################################################

proc get_snmp_whole_oid_from_all_index_query { snmp_raw_query } {

    global verbose

    regexp {oid \{([0-9/.]+)\}} $snmp_raw_query dummy snmp_oid
    if {$verbose == 1} {
	puts "get_snmp_whole_oid_from_all_index_query:"
	puts "snmp_oid is $snmp_oid"
    }

    return $snmp_oid
}

###############################################################################
#
#   Description: Get snmp static pid from whole oid in one index table query
#                This is a utilitly function to get static oid from whole oid
#                at one index table.
#                The function is not currently used. 
#
#   IN: Snmp oid for one index table query.
#   OUT: Snmp static oid
#
#
###############################################################################

proc get_snmp_static_oid_from_whole_oid_for_1_index_table { snmp_oid } {

    global verbose

    regexp {(.*)\.([0-9]*)$} $snmp_oid dummy snmp_static_oid

    if {$verbose == 1} {
	puts "get_snmp_static_oid_from_whole_oid_for_1_index_table: "
	puts "snmp_static_oid is $snmp_static_oid"
    }

    return $snmp_static_oid
}

###############################################################################
#
#   Description: get snmp static oid from whole query for one index table
#
#   IN: snmp raw query
#   OUT: snmp value
#
###############################################################################

proc get_snmp_static_oid_from_1_index_query { snmp_raw_query } {

    global verbose

    regexp {oid \{(.*)\.([0-9]+)\}} $snmp_raw_query dummy \
	    snmp_static_oid

    if {$verbose == 1} {
	puts "get_snmp_static_oid_from_1_index_query: "
	puts "snmp_static_oid is $snmp_static_oid"
    }

    if { $snmp_static_oid != ""} {
	return $snmp_static_oid
    }
}

###############################################################################
#
#   Description: Get snmp static oid from whole query for 2 index tables
#
#   IN: Snmp raw query
#   OUT: Snmp value
#
###############################################################################

proc get_snmp_static_oid_from_2_index_query { snmp_raw_query } {

    global verbose

    regexp {oid \{(.*)\.([0-9]+)\.([0-9]+)\}} $snmp_raw_query dummy \
	    snmp_static_oid

    if {$verbose == 1} {
	puts "get_snmp_static_oid_from_2_index_query: "
	puts "snmp_static_oid is $snmp_static_oid"
    }

    if { $snmp_static_oid != ""} {
	return $snmp_static_oid
    }
}

###############################################################################
#
#   Description: Get snmp query last index from whole oid
#                The oid can be from one index or 2 index table. 
#                For one index tale, it is the index. For 2 
#                index table, it is the second index.
#                This is an utility function and currently is not being used. 
#
#   IN: snmp raw query
#   OUT: snmp value
#
###############################################################################

proc get_snmp_oid_last_index_from_oid { snmp_oid } {
    
    global verbose

    regexp {(.*)\.([0-9]*)$} $snmp_oid dummy dummy snmp_oid_last_index

    if {$verbose == 1} {
	puts "get_snmp_oid_last_index_from_oid: "
	puts "snmp_oid_last_index is $snmp_oid_last_index"
    }

    return $snmp_oid_last_index
}

###############################################################################
#
#   Description: Get snmp first index from whole query.at 2 index table
#
#   IN: snmp raw query
#   OUT: snmp value
#
###############################################################################

proc get_snmp_oid_first_index { snmp_raw_query } {
    
    global verbose

    regexp {oid (.*)\.([0-9]+)\.([0-9]+)\}} $snmp_raw_query dummy dummy1 \
	    snmp_oid_first_index

    if {$verbose == 1} {
	puts "get_snmp_oid_first_index: "
	puts "snmp_oid_first_index is $snmp_oid_first_index"
    }

    if { $snmp_oid_first_index != "" } {
	return $snmp_oid_first_index
    }
}


###############################################################################
#
#   Description: Get snmp query last index from whole query in one index or
#                2 index table. For one index tale, it is the index. For 2 
#                index table, it is the second index. 
#
#   IN: snmp raw query
#   OUT: snmp value
#
###############################################################################

proc get_snmp_oid_last_index { snmp_raw_query } {
    
    global verbose

    regexp {oid (.*)\.([0-9]+)\}} $snmp_raw_query dummy dummy \
	    snmp_oid_last_index

    if {$verbose == 1} {
	puts "get_snmp_oid_last_index: "
	puts "snmp_oid_last_index is $snmp_oid_last_index"
    }

    if { $snmp_oid_last_index != "" } {
	return $snmp_oid_last_index
    }
}

###############################################################################
#
#   Description: Get snmp query value from whole snmp query. 
#
#   IN: snmp raw query
#   OUT: snmp value
#
###############################################################################

proc get_snmp_query_value { snmp_raw_query } {

    global verbose

    regexp {value \{(.*)\}} $snmp_raw_query dummy snmp_value

    return $snmp_value
}

###############################################################################
#
#    Description: find_index_from_one_index_table
#                 Given static oid and a query value to find its corresponding
#                 index value.in one index table.
#
#                 This function go through a while loop to find an index
#                 which has query value matching with the given value in input
#
#    In: query match value, static oid 
#    Out: index value for the matching query
#
###############################################################################

proc find_index_from_one_index_table { match_value static_oid } {
        
    global verbose
    set query_value 0
    set Index 0
    set Index_for_the_interface ""

    while { $query_value != $match_value} {

        set query [sys_reqinfo_snmp oid $static_oid.$Index get_type next]

        if {$verbose == 1} {
	    puts "find_index_from_one_index_table: "
            puts "query is $query"
        }

	set static_oid_for_the_query [get_snmp_static_oid_from_1_index_query \
		$query]

	# If the query oid is same as next static, it means there is no match
	# is found.

	if { $static_oid_for_the_query != $static_oid } {
	    if {$verbose == 1} {
                puts "find_index_from_one_index_table: "
                puts "The query has come to a new static oid and no match"
                puts "is found"
	    }
	    break;
	}

	set Index [get_snmp_oid_last_index $query]

	if {$verbose == 1} {
	    puts "find_index_from_one_index_table: "
            puts "Index is $Index"
        }

        set query_value [get_snmp_query_value $query]

        if { $static_oid == "1.3.6.1.4.1.9.9.166.1.8.1.1.1" } {
            if [regexp {Match ([^ ]*)} $query_value dummy query_value] {
                if {$verbose == 1} {
                    puts "query_value is $query_value"
                }
            }
        }

        if { $query_value == $match_value} {
            if {$verbose == 1} {
		puts "find_index_from_one_index_table: "
                puts "Found the instance we want for the given value"
            }
            set Index_for_the_interface  $Index
            break;
        }

    }

    return $Index_for_the_interface

}

###############################################################################
#
#    Description: find_policy_index_from_one_index_table
#                 Given static oid and a query value to find its corresponding
#                 index value.in one index table.
#
#                 This function goes through a while loop to find an index
#                 which has query value matching with the given value in input
#
#    In: query match value, static oid direction
#    Out: index value for the matching query
#
###############################################################################

proc find_policy_index_from_one_index_table { match_value dir } {
        
    global verbose
    set query_value 0
    set Index 0
    set Index_for_the_interface ""

    # cbQosIndex oid ( 1.3.6.1.4.1.9.9.166.1.1.1.1.4 ) is not supported 
    # by snmp, use integer oid to for its snmp query.
 
    set cbQosIfIndex_oid "1.3.6.1.4.1.9.9.166.1.1.1.1.4"
    set static_oid $cbQosIfIndex_oid
    
    while { 1 } {

        set query [sys_reqinfo_snmp oid $static_oid.$Index get_type next]

        if {$verbose == 1} {
            puts "query is $query"
        }

        set static_oid_for_the_query [get_snmp_static_oid_from_1_index_query \
                $query]

        # If the query oid is same as next static, it means there is no match
        # is found.

        if { $static_oid_for_the_query != $static_oid } {
            if { $verbose == 1 } {
                puts "find_policy_index_from_one_index_table: "
                puts "The query comes to a new static oid and no match"
            }
            break;
        }

        set Index [get_snmp_oid_last_index $query]

        if { $verbose == 1 } {
            puts "find_policy_index_from_one_index_table: Index is $Index"
        }

        set query_value [get_snmp_query_value $query]

        if { $query_value == $match_value} {
 
            set cbQosPolicyDirection_oid "1.3.6.1.4.1.9.9.166.1.1.1.1.3"
            set direction_value [get_query_value_from_whole_oid \
                    $cbQosPolicyDirection_oid.$Index]

            set dir_match 0

            if { $dir  == "input" } {
                if { $direction_value == 1 } {
                    set dir_match 1		
                }
            }

            if { $dir == "output" } {
                if { $direction_value == 2 } {
                    set dir_match 1		
                }
            }

            if { $dir_match == 1 } {
                set Index_for_the_interface $Index
                break;
            }
        }
    }

    return $Index_for_the_interface

}

###############################################################################
#
#    Description: find_2_indexes_from_2_index_table
#                 This function go through a while loop for a 2 index table
#                 query and find the 2 indexes query which has query value
#                 matching with the given value
#
#    IN: query value, static oid
#    OUT: 2 indexes
#
###############################################################################

proc find_2_indexes_from_2_index_table { match_value static_oid } {

    global verbose
    set query_value 0
    set Index1 0
    set Index2 0
    set Index_for_the_interface ""

    while { $query_value != $match_value} {
        set query [sys_reqinfo_snmp oid $static_oid.$Index1.$Index2 \
		get_type next]

        if {$verbose == 1} {
	    puts "find_2_indexes_from_2_index_table: "
            puts "query is $query"
        }

	set static_oid_for_this_query \
                [get_snmp_static_oid_from_2_index_query $query]

	# If the query oid is same as next one instead of this one, it means 
	# there is no match is found.
	if { $static_oid_for_this_query != $static_oid } {
	    if {$verbose == 1} {
		puts "find_2_indexes_from_2_index_table: "
		puts "The query search has come to a new oid and no match is "
		puts "found"
	    }
	    break;
	}

	set Index1 [get_snmp_oid_first_index $query]
	set Index2 [get_snmp_oid_last_index $query]

	if {$verbose == 1} {
	    puts "find_2_indexes_from_2_index_table: "
            puts "Index1 is $Index1 Index2 is $Index2"
        }

        set query_value [get_snmp_query_value $query]

        if { $query_value == $match_value} {
            if {$verbose == 1} {
		puts "find_2_indexes_from_2_index_table: "
                puts "We found the 2 indexes we want since we see a match "
		puts "query value"
            }
            set Index_for_the_interface  $Index1
	    lappend Index_for_the_interface $Index2
            break;
        }
    }

    return $Index_for_the_interface

}

##############################################################################
#
#    Description: find_2nd_index_from_2_index_table
#                 This function go through a while loop for a 2 index table
#                 query and find the 2nd indexe query which has query value
#                 matching with the given value
#
#    IN: query value, first index, static oid
#    OUT: 2nd indexes
#
##############################################################################

proc find_2nd_index_from_2_index_table { match_value first_index \
        static_oid } {

    global verbose
    set query_value 0
    set Index2 0
    set Index_for_the_interface ""

    while { 1 } {

        set query [sys_reqinfo_snmp oid $static_oid.$first_index.$Index2 \
		get_type next]

        if {$verbose == 1} {
            puts "find_2nd_index_from_2_index_table: "
            puts "query is $query"
        }

        set static_oid_for_this_query \
                [get_snmp_static_oid_from_2_index_query $query]

        # If the query oid is same as next one instead of this one, it means 
        # there is no match is found.

        if { $static_oid_for_this_query != $static_oid } {
            if {$verbose == 1} {
                puts "find_2_indexes_from_2_index_table: "
                puts "The query search comes to a new oid and no match"
            }
            break;
        }

        set Index1 [ get_snmp_oid_first_index $query ]

        # When checkinig if a classmap in a policymap of an interface, 
        # Needs to check its policyindex only.

        if { $Index1 != $first_index } {
            break;
        }

        set Index2 [get_snmp_oid_last_index $query]

        set query_value [get_snmp_query_value $query]

        if { $query_value == $match_value} {
            if {$verbose == 1} {
                puts "find_2nd_indexes_from_2_index_table: "
                puts "Index2 is $Index2 "
            }
            # If the object oid is cbQosParentObjectsIndex, one parent
            # can have multiple children, need to continuously search
            # to find all children object indexes
            if { $static_oid == "1.3.6.1.4.1.9.9.166.1.5.1.1.4" } {
                if { $Index_for_the_interface == "" } {
                    set Index_for_the_interface $Index2
                } else {
                    lappend Index_for_the_interface $Index2
                }
            } else {
                return $Index2
            }
        }
    }

    return $Index_for_the_interface

}

##############################################################################
#
#   Description: Config ios command at config mode. 
#  
#   IN: ios configuration cli
#   OUT: NONE
#
##############################################################################

proc ios_config { config_cmd } {

    if [catch {cli_open} result] {
        error $result "Cannot open cli"
    } else {
	array set cli1 $result
    }


    if [catch {cli_exec $cli1(fd) "enable"} result] {
        error $result "Cannot go to enable mode"
    }


    if [catch {cli_exec $cli1(fd) "conf t"} result] {
        error $result "Cannot go to configuration mode"
    } else {
	if [catch {cli_exec $cli1(fd) $config_cmd} result] {
            error $result "Error in config command"
	}
    }

    if [catch {cli_exec $cli1(fd) "end"} result] {
        error $result "End of the cli"
    }

    if [catch {cli_close $cli1(fd) $cli1(tty_id)} result] {
        error $result "Close cli"
    }
}


###############################################################################
#
#    Description: Perform IOS configuration command to enable snmp server
#                 In order to get snmp query from the snmp database. 
#
#    In: None. 
#    Out: None.
#
##############################################################################

proc enable_snmp_server {} {

    # In order to get snmp infor for the script, the snmp agent has to be set
    # up first, so the snmp data base is available to take info from there. 

    set enable_snmp_cli "snmp-server community public RW"
    ios_config $enable_snmp_cli

}

###############################################################################
#
#    Description: get_query_value_from_whole_oid
#                 get query value by static oid, index1 and index2 
#                 The query value can be a packet counter or drops
#
#    In: static oid, index1, index2
#    Out: query value
#
#
###############################################################################

proc get_query_value_from_whole_oid { whole_oid } {

    global verbose

    set whole_query [sys_reqinfo_snmp oid $whole_oid get_type exact]

    if {$whole_query != ""} {
	set query_value [get_snmp_query_value $whole_query]
	if {$verbose == 1} {
	    puts "get_query_value_from_whole_oid: "
	    puts "query_value is $query_value"
	}
	return $query_value
    } else {
	if {$verbose == 1} {
	    puts "get_query_value_from_whole_oid: "
	    puts "This counter is not supported, there is no value to be given"
	}
    }
}

###############################################################################
#
#    Description: get_query_value 
#                 get query value by static oid, index1 and index2 
#                 The query value can be a packet counter or drops
#                 This is utility function, we might not need it. 
#
#    In: static oid, index1, index2
#    Out: query value
#
#
###############################################################################

proc get_query_value_from_oid { static_oid Index1 Index2 } {
    
    global verbose

    set pkt_counter_query [sys_reqinfo_snmp oid \
	    $static_oid.$Index1.$Index2 get_type exact]

    if {$pkt_counter_query != ""} {
	set pkt_counter_value [get_snmp_query_value $pkt_counter_query]
	if {$verbose == 1} {
	    puts "get_query_value_from_oid: "
	    puts "pkt_counter_value is $pkt_counter_value"
	}
	return $pkt_counter_value
    } else {
	if {$verbose == 1} {
	    puts "get_query_value_from_oid: "
	    puts "This counter is not supported, could not found a value "
	    puts "for the query"
	}
    }
}

###############################################################################
#
# Description: classmap_counter_report
#              Report classmap drop counters by the interface's PolicyIndex 
#              and ObjectIndex. 
#              One interface can have multiple classmaps, so one interface can
#              have multiple objectIndex for its corresponding classmaps and 
#              drop counters.
#
# In:  PolicyIndex ObjectsIndex
# Out: Report classmap drop counters and pre policy and post policy counters.
#
###############################################################################

proc classmap_counter_report { PolicyIndex  ObjectsIndex } {

    global verbose

    # cbQosCMDropPkt (1.3.6.1.4.1.9.9.166.1.15.1.1.13) is a 
    # unsupported snmp name in eem, so integer oid has to be used here.
    set cbQosCMDropPkt_oid "1.3.6.1.4.1.9.9.166.1.15.1.1.13"
    set cbQosCMDropPkt_value [get_query_value_from_whole_oid \
	    $cbQosCMDropPkt_oid.$PolicyIndex.$ObjectsIndex]

    lappend classmap_drop_counters $cbQosCMDropPkt_value

    # cbQosCMPrePolicyByte_query is not a supported snmp name, needs to
    # use its integer oid 1.3.6.1.4.1.9.9.166.1.15.1.1.5
    set cbQosCMPrePolicyByte_oid "1.3.6.1.4.1.9.9.166.1.15.1.1.5"
    set cbQosCMPrePolicyByte_value [get_query_value_from_whole_oid \
	    $cbQosCMPrePolicyByte_oid.$PolicyIndex.$ObjectsIndex]
    if {$verbose == 1} {
	puts "classmap_counter_report: "
	puts "cbQosCMPrePolicyByte_value is $cbQosCMPrePolicyByte_value"
    }

    lappend classmap_drop_counters $cbQosCMPrePolicyByte_value

    # cbQosCMPrePolicyByte_query is not a supported snmp name, needs to 
    # use its integer oid 1.3.6.1.4.1.9.9.166.1.15.1.1.9
    set cbQosCMPostPolicyByte_oid "1.3.6.1.4.1.9.9.166.1.15.1.1.9"
    set cbQosCMPostPolicyByte_value [get_query_value_from_whole_oid \
	    $cbQosCMPostPolicyByte_oid.$PolicyIndex.$ObjectsIndex]
    if {$verbose == 1} {
	puts "classmap_counter_report: "
	puts "cbQosCMPostPolicyByte_value is $cbQosCMPostPolicyByte_value"
    }
    lappend classmap_drop_counters $cbQosCMPostPolicyByte_value

    if {$verbose == 1} {
	puts "classmap_counter_report: "
	puts "classmap_drop_counters is $classmap_drop_counters"
    }

    if {$classmap_drop_counters != ""} {
	return $classmap_drop_counters
    }
}

###############################################################################
#
# Description: get_CM_output
#              Report classmap output
#
# In:  cbQosCMDropPkt_value cbQosCMPrePolicyByte_value 
# Out: Report classmap drop counters and pre policy and post policy counters.
#
###############################################################################

proc get_CM_output { cbQosCMName_value cbQosCMInfo_str_value \
        cbQosCMDropPkt_value cbQosCMPrePolicyByte_value \
        cbQosCMPostPolicyByte_value MatchStmt } {

    set classmap_output "\nCLASSMAP  <name>: $cbQosCMName_value \
		    <description>: $cbQosCMInfo_str_value "
    append classmap_output "  <drop packet>: $cbQosCMDropPkt_value"
    append classmap_output "  <pre-policybyte>: $cbQosCMPrePolicyByte_value"
    append classmap_output "  <post-policybyte>: $cbQosCMPostPolicyByte_value"
    append classmap_output "  <match statement>: $MatchStmt"

    return $classmap_output

}


###############################################################################
#
# Description: get_MatchStmt_output
#              Report MatchStmt output
#
# In:  name PrePolicyPkt
# Out: Report classmap drop counters and pre policy and post policy counters.
#
###############################################################################

proc get_MatchStmt_output { name PrePolicyPkt } {

    set matchstatement_output "\nMATCHSTMT  <name>:"
    append matchstatement_output $name         
    append matchstatement_output "  <pre-policy packet>:"
    append matchstatement_output " $PrePolicyPkt"

    return $matchstatement_output
}
                    
###############################################################################
#
#  Description: get_ifindex_no
#               get interface index number from snmp agent
# 
#  IN:          Interface number
#  OUT:         interface index no.
#
###############################################################################

proc get_ifindex_no { intf } {

    global verbose

    set ios_cli "show run | i snmp-server community public RW"
    set cli_out [ios_enable_command $ios_cli]

    if ![regexp {snmp-server community public RW} $cli_out] {
        enable_snmp_server
    }

    # Get ifIndex from cli "show snmp mib ifmib ifIndex "
    set show_snmp_mib_cli "show snmp mib ifmib ifIndex $intf"
    set cli_out [ios_enable_command $show_snmp_mib_cli]
    if [regexp {Invalid input detected at} $cli_out ] {
        puts "You do not have a valid interface in your inputfile, needs"
        puts "to correct it ............................................"
        exit
    }

    # Get interface name, because value getten from snmp is only 2 characters, 
    # so we need to substitute the whole string to 2 characters.

    if [regexp {.* Ifindex = ([0-9]*)} $cli_out dummy ifindex_no] {
        if {$verbose == 1} {
            puts "get_ifindex_no: ifindex_no is $ifindex_no"
        }
    } elseif [regexp {Invalid ifIndex for} $cli_out ] {
        puts "You do not have a valid interface in your inputfile, the result"
        puts "for \"show snmp ifmib infIndex $intf\" is \"$cli_out\""
        exit
    }

    return $ifindex_no
}

##############################################################################
#
#  Descriptiong: get_PolicyMapName_from_ConfigIndex
#                Get policymap name from its config index
#
#  IN:           cbQosPolicyIndex
#  OUT:          return policymap name corresponding to the configIndex
#
##############################################################################

proc get_PolicyMapName_by_ConfigIndex { cbQosConfigIndex } {

    global verbose

    # cbQosPolicyMapName (1.3.6.1.4.1.9.9.166.1.6.1.1.1) is Not a supported 
    # snmp name by eem. We have to use integer oid to get the query.

    set cbQosPolicyMapName_oid "1.3.6.1.4.1.9.9.166.1.6.1.1.1"
    
    set cbQosPolicyMapName_query [sys_reqinfo_snmp oid \
            $cbQosPolicyMapName_oid.$cbQosConfigIndex get_type exact]

    set cbQosPolicyMapName [get_snmp_query_value $cbQosPolicyMapName_query]

    return $cbQosPolicyMapName

}


##############################################################################
#
#  Descriptiong: get_PolicyMapName_from_PolicyIndex
#                Get policymap name from its config index
#
#  IN:           cbQosPolicyIndex
#  OUT:          return policymap name corresponding to the configIndex
#
##############################################################################

proc get_PolicyMapName_by_PolicyIndex { cbQosPolicyIndex } {

    global verbose

    set cbQosObjectsIndex $cbQosPolicyIndex
    
    # Get PolicyMapeName config index first
    set cbQosConfigIndex_oid "1.3.6.1.4.1.9.9.166.1.5.1.1.2"
    set cbQosConfigIndex_query [sys_reqinfo_snmp oid \
            $cbQosConfigIndex_oid.$cbQosPolicyIndex.$cbQosObjectsIndex \
            get_type exact]

    set cbQosConfigIndex [get_snmp_query_value $cbQosConfigIndex_query]

    set cbQosPolicyMapName [get_PolicyMapName_by_ConfigIndex \
            $cbQosConfigIndex]

    if {$verbose == 1} {
        puts "get_PolicyMapName_by_PolicyIndex:\
                cbQosPolicyMapName is $cbQosPolicyMapName"
    }

    return $cbQosPolicyMapName

}

##############################################################################
#
#  Descriptiong: get_CMInfo
#                Get CMInfo from its config index
#
#  IN:           config Index
#  OUT:          return CMInfo corresponding to the configIndex
#
##############################################################################

proc get_CMInfo { cbQosConfigIndex } {

    global verbose

    # Get classmap info. cbQosCMInfo (1.3.6.1.4.1.9.9.166.1.7.1.1.3)
    # is not a supported snmp in eem. So integer oid needs to be used.

    set cbQosCMInfo_oid "1.3.6.1.4.1.9.9.166.1.7.1.1.3"
    set cbQosCMInfo_query [sys_reqinfo_snmp oid \
            $cbQosCMInfo_oid.$cbQosConfigIndex get_type exact]

    set cbQosCMInfo_query_value [get_snmp_query_value \
            $cbQosCMInfo_query]

    if {$verbose == 1} {
        puts "get_CMInfo:\
                cbQosCMInfo_query_value is $cbQosCMInfo_query_value"
    }

    if { $cbQosCMInfo_query_value == "2"} {
        set cbQosCMInfo_value "matchAll"
    }

    if { $cbQosCMInfo_query_value == "3"} {
        set cbQosCMInfo_value "matchAny"
    }

    return $cbQosCMInfo_value
}

##############################################################################
#
#  Descriptiong: get_MatchStmtName
#                Get match statement name from its config index
#
#  IN:           config Index
#  OUT:          return match statment name corresponding to the configIndex
#
##############################################################################

proc get_MatchStmtName { cbQosConfigIndex } {

    global verbose

    set cbQosMatchStmtName_oid "1.3.6.1.4.1.9.9.166.1.8.1.1.1"
    set cbQosMatchStmtName_query [sys_reqinfo_snmp oid \
            $cbQosMatchStmtName_oid.$cbQosConfigIndex \
            get_type exact]
    set cbQosMatchStmtName_value [get_snmp_query_value \
            $cbQosMatchStmtName_query]

    regexp {Match (.*)} $cbQosMatchStmtName_value dummy \
            cbQosMatchStmtName_str_value
    if {$verbose == 1} {
        puts "get_MatchStmtName: cbQosMatchStmtName_str_value is\
                $cbQosMatchStmtName_str_value"
    }

    return $cbQosMatchStmtName_str_value
}

##############################################################################
#
#  Descriptiong: get_MatchStmt
#                Get match statement from classmap configured on an interface
#
#  IN:           classmap config index value
#  OUT:          return match statment attacted to this classmap
#
##############################################################################

proc get_MatchStmt { cbQosPolicyIndex CM_ConfigIndex } {

    global verbose

    # step 1:
    # From CM_ConfigIndex, we should be able to get cbQosObjectsIndex
    # for the classmap
    set cbQosMatchStmtName ""
    
    set cbQosConfigIndex_oid "1.3.6.1.4.1.9.9.166.1.5.1.1.2"
    set cbQosObjectsIndex_classmap [find_2nd_index_from_2_index_table \
            $CM_ConfigIndex $cbQosPolicyIndex $cbQosConfigIndex_oid]

    # step 2:
    # After we know objectsIndex for the classmap, we can get its child
    # since the classmap is the parent of the matchstatement 

    set cbQosParentObjectsIndex_oid "1.3.6.1.4.1.9.9.166.1.5.1.1.4"
    set cbQosObjectsIndex_ms [find_2nd_index_from_2_index_table \
            $cbQosObjectsIndex_classmap $cbQosPolicyIndex \
            $cbQosParentObjectsIndex_oid]

    if { $verbose == 1 } {
        puts "cbQosObjectsIndex_ms is $cbQosObjectsIndex_ms"
    }

    foreach cbQosObjectsIndex [split $cbQosObjectsIndex_ms " "] {

        if { $verbose == 1 } {
            puts "cbQosPolicyIndex is $cbQosPolicyIndex"
            puts "cbQosObjectsIndex is $cbQosObjectsIndex"
        }

        # step 3:
        # check if it is match statement

        set cbQosObjectsType_oid "1.3.6.1.4.1.9.9.166.1.5.1.1.3"
        set cbQosObjectsType_query [sys_reqinfo_snmp oid \
                $cbQosObjectsType_oid.$cbQosPolicyIndex.$cbQosObjectsIndex \
                get_type exact]
        if {$verbose == 1} {
            puts "cbQosObjectsType_query = $cbQosObjectsType_query"
        }
        set cbQosObjectsType_value [get_snmp_query_value \
                $cbQosObjectsType_query]

        if { $cbQosObjectsType_value == 3 } {

            # Step 4:
            # This would be objectIndex for the match statement, we can get 
            # ConfigIndex for the match statement

            set query [sys_reqinfo_snmp oid \
                    $cbQosConfigIndex_oid.$cbQosPolicyIndex.$cbQosObjectsIndex \
                    get_type exact]

            if { $verbose == 1 } {
                puts "query is $query"
            }
        
            set cbQosConfigIndex_value [get_snmp_query_value $query]

            if { $verbose == 1 } {
                puts "cbQosConfigIndex_value is $cbQosConfigIndex_value"
            }

            # step 5:
            # From configIndex, we can get the match statement

            set cbQosMatchStmtName_str_value \
                    [ get_MatchStmtName $cbQosConfigIndex_value ]

            if { $verbose == 1 } {
                puts "cbQosMatchStmtName_str_value = \
                        $cbQosMatchStmtName_str_value"
            }

            if { $cbQosMatchStmtName == "" } {
                set cbQosMatchStmtName $cbQosMatchStmtName_str_value
            } else {
                lappend cbQosMatchStmtName $cbQosMatchStmtName_str_value
            }

            if { $verbose == 1 } {
                puts "cbQosMatchStmtName = $cbQosMatchStmtName"
            }
            return $cbQosMatchStmtName
        }       
    }
    
    return $cbQosMatchStmtName

}
##############################################################################
#
#  Descriptiong: get_intf_policy_msg
#                Get syslog message for policymap, interfce and direction case
#
#  IN:           intf: intface name 
#                dir_from_input_file: policy direction
#                cbQosPolicyMapName: policymap name
#  OUT:          General syslog message
#
##############################################################################

proc get_intf_policy_msg { intf dir cbQosPolicyMapName } {

    global verbose

    set msg "\nInterface $intf, service policy direction \"$dir\" "
    append msg "on policymap \"$cbQosPolicyMapName\":"

    return $msg

}

##############################################################################
#
#  Descriptiong: get_diff_only_msg
#                Get syslog message for diff_only case
#
#  IN:           intf: intface name 
#                dir_from_input_file: policy direction
#                cbQosPolicyMapName: policymap name
#                cbQosCMName_value: classmap name
#                MatchStmt: matchstatment corresponding the classmap
#                newdrop: during the 15 minutes
#  OUT:          syslog message for diff only
#
##############################################################################

proc get_diff_only_msg { intf dir cbQosPolicyMapName cbQosCMName_value \
        MatchStmt newdrop } {

    global verbose

    set msg [get_intf_policy_msg $intf $dir $cbQosPolicyMapName]

    append msg \n "New drop during this 15 minutes for classmap "
    append msg "$cbQosCMName_value is $newdrop, its match statement is "
    append msg "\"$MatchStmt\""

    return $msg

}


##############################################################################
#
#  Descriptiong: get_all_drops_for_the_interface
#                Get all drops for the interface at the direction user 
#                specified. This is all drops for one service policy when
#                one service policy is corresponding to a direction.
# 
#
#  IN:           Interface number, direction for the interface (in and out)
#  OUT:          All drops counters for this interface on this direction as
#                customer specified.
#
##############################################################################


proc get_all_drops_for_the_interface { diff_only intf dir drop_file } {

    # In order to get drop packets from mib statistics, such as classmap drop
    # counters, we need to get their table indexes first, which are
    # cbQosPolicyIndex (to tell which policy the counter is for, this will
    # specify which policy the interface would have) and cbQosObjectsIndex
    # (to tell which object it is, this would specify if it is classmap or 
    # match statement for the interface). 

    # For getting cbQosPolicyIndex, we can use cbQosServicePolicyTable.
    # The table has oid 1.3.6.1.4.1.9.9 and one table index cbQosPolicyIndex
    # By given interface name and interface index number, we can get 
    # cbQosPolicyIndex from it.

    global verbose

    set msg ""

    # We can get the inIndex from 2 methords,
    # One is from existing ios show cli command.
    # The other is from eem IF-MIB

    set ifindex_no [get_ifindex_no $intf]

    # After get ifIndex from above use  sys_reqinfo_snmp TCL extension cli
    # to get PolicyIndex

    set cbQosPolicyIndex [find_policy_index_from_one_index_table $ifindex_no \
	    $dir]

    if {$verbose == 1} {
	puts "get_all_drops_for_the_interface: "
	puts "cbQosPolicyIndex is $cbQosPolicyIndex"
    }

    # After getting cbQosPolicyIndex, check if its policy direction is what
    # user wants. if it is, continuously go on. If it is not, return to
    # not report anything. 

    set cbQosPolicyDirection_oid "1.3.6.1.4.1.9.9.166.1.1.1.1.3"
    set direction_value [get_query_value_from_whole_oid \
	    $cbQosPolicyDirection_oid.$cbQosPolicyIndex]

    set dir_match 0

    if { $dir == "input" } {
        if { $direction_value == 1 } {
	    set dir_match 1		
	}
    }

    if { $dir == "output" } {
        if { $direction_value == 2 } {
	    set dir_match 1		
	}
    }

    if {$verbose == 1} {
        puts "get_all_drops_for_the_interface: dir is $dir"
	puts "direction_value is $direction_value"
    }
   
    if { $dir_match == 1 } {
	if { $verbose == 1 } {
	    puts "get_all_drops_for_the_interface: "
	    puts "There is a match between user policy direction and "
	    puts "user request, so continuously go"
	}
    } else {
	if { $verbose == 1 } {
	    puts "get_all_drops_for_the_interface: "
	}
	puts "There is No match between user policy direction and "
	puts "user request on the inteface $intf, stop here and return"
	puts "to check next query"

	return
    }

    # Now we need to find out cbQosObjectsIndex in order to find a drop 
    # corresponding to it.
    # cbQosObjectsEntry for cbQosConfigIndex, cbQosObjectsType, 
    # cbQosParentObjectsIndex has 2 table indexes: cbQosPolicyIndex and 
    # cbQosbObjectsIndex
    # Let us verify it to see if it is the policymap we configured.
    # cbQosConfigIndex (1.3.6.1.4.1.9.9.166.1.5.1.1.2) is not a snmp name 
    # supported by eem, we have to use integer oid to get the query.

    # Verify if the cbQosPolicyMapName from mib is in "show run int $intf"
    # For policymap, its cbQosObjectsIndex is same as its cbQosPolicyIndex

    set cbQosObjectsIndex $cbQosPolicyIndex

    # cbQosObjectsType (1.3.6.1.4.1.9.9.166.1.5.1.1.3) is Not a snmp name 
    # supported# by eem so oid integer has to be used here.
    set cbQosObjectsType_oid "1.3.6.1.4.1.9.9.166.1.5.1.1.3"

    set cbQosObjectsType_query [sys_reqinfo_snmp oid \
	    $cbQosObjectsType_oid.$cbQosPolicyIndex.$cbQosObjectsIndex \
	    get_type exact]
    if {$verbose == 1} {
	puts "cbQosObjectsType_query = $cbQosObjectsType_query"
    }
    set cbQosObjectsType_value [get_snmp_query_value $cbQosObjectsType_query]


    while { $cbQosObjectsType_value } {

	if {$verbose == 1} {
	    puts "get_all_drops_for_the_interface: "
	    puts "cbQosPolicyIndex is $cbQosPolicyIndex"
	    puts "cbQosObjectsIndex is $cbQosObjectsIndex"
	}

	set cbQosConfigIndex_oid "1.3.6.1.4.1.9.9.166.1.5.1.1.2"
	set cbQosConfigIndex_query [sys_reqinfo_snmp oid \
		$cbQosConfigIndex_oid.$cbQosPolicyIndex.$cbQosObjectsIndex \
		get_type exact]

	if {$verbose == 1} {
	    puts "get_all_drops_for_the_interface: "
	    puts "cbQosConfigIndex_query is $cbQosConfigIndex_query"
	}
	
	set cbQosConfigIndex_value \
		[get_snmp_query_value $cbQosConfigIndex_query]

	if {$verbose == 1} {
	    puts "get_all_drops_for_the_interface: "
	    puts "cbQosConfigIndex_value is $cbQosConfigIndex_value"
	}

	# If policymap value is 1, it means it is a policymap, then we go to 
	# check policymap config map.

	if { $cbQosObjectsType_value == 1 } {

            set cbQosPolicyMapName [get_PolicyMapName_by_ConfigIndex \
                    $cbQosConfigIndex_value]

            if { $diff_only == "no" } {
                append msg [get_intf_policy_msg $intf $dir $cbQosPolicyMapName]
	    }
	} 

	if { $cbQosObjectsType_value == 2 } {

	    # If cbQosObjectsType value is 2, it means it is a classmap, we 
	    # will check its class map by configIndex value configuration and
	    # decide its drop value.

	    set cbQosCMName "1.3.6.1.4.1.9.9.166.1.7.1.1.1"
	    set cbQosCMName_query [sys_reqinfo_snmp oid \
		    $cbQosCMName.$cbQosConfigIndex_value get_type exact]
	    set cbQosCMName_value [get_snmp_query_value $cbQosCMName_query]

	    if {$verbose == 1} {
		puts "get_all_drops_for_the_interface: "
		puts "cbQosCMName_value is $cbQosCMName_value"
	    }
	    
            set cbQosCMInfo_str_value [get_CMInfo $cbQosConfigIndex_value]

            set classmap_counters [classmap_counter_report $cbQosPolicyIndex \
                    $cbQosObjectsIndex]

            if { $classmap_counters  != 0 } {
                set cbQosCMDropPkt_value [lindex $classmap_counters 0]
                set cbQosCMPrePolicyByte_value [lindex $classmap_counters 1]
                set cbQosCMPostPolicyByte_value [lindex $classmap_counters 2]
	    }

            # John gives EFT feedback to add classmap match statement to the 
            # to syslog make user easy to see

            set PolicyMapName [get_PolicyMapName_by_PolicyIndex \
                    $cbQosPolicyIndex ]

            # The matchstatement here can be multiple values since one
            # classmap parent can have multiple children

            set MatchStmt [get_MatchStmt $cbQosPolicyIndex \
                    $cbQosConfigIndex_value]

            set classmap_output [get_CM_output \
                    $cbQosCMName_value $cbQosCMInfo_str_value \
                    $cbQosCMDropPkt_value $cbQosCMPrePolicyByte_value \
                    $cbQosCMPostPolicyByte_value $MatchStmt]

            if { $diff_only == "no" } {
                append msg " $classmap_output"
	    }

	    if [file exists $drop_file] {
		set drop_fileId [open "$drop_file" r 0777]
		set run_again "yes"
		set droplist ""
		set current_cbQos_logged 0

		while {$run_again == "yes"} {
		    gets $drop_fileId value
		    if {$verbose == 1} {
			puts "get_all_drops_for_the_interface: "
			puts "The line in existing file is $value"
		    }
		    if {$value != "" } {
			set pre_cbQosPolicyIndex [lindex $value 0]
			set pre_cbQosObjectsIndex [lindex $value 1]
			set pre_cbQosCMDropPkt_value [lindex $value 2]
			set pre_cbQosCMPrePolicyByte_value [lindex $value 3]
			set pre_cbQosCMPostPolicyByte_value [lindex $value 4]

			if { $pre_cbQosPolicyIndex == $cbQosPolicyIndex } {
			    if { $pre_cbQosObjectsIndex == \
				    $cbQosObjectsIndex } {

				set newdrop [expr {$cbQosCMDropPkt_value - \
                                        $pre_cbQosCMDropPkt_value} ]

				if { $newdrop > 0 } {  
                                    if { $diff_only == "yes" } {

                                        set msg [ get_diff_only_msg \
                                                $intf $dir \
                                                $cbQosPolicyMapName \
                                                $cbQosCMName_value \
                                                $MatchStmt \
                                                $newdrop ]

                                    }
				}

				# The output file result is:
				# PolicyIndex $ObjectsIndex
				# $cbQosCMDropPkt_value 
				# $cbQosCMPrePolicyByte_value
				# $cbQosCMPostPolicyByte_value
				if {$droplist !=""} {
				    append droplist "\n"
				}
				append droplist "$cbQosPolicyIndex"
				append droplist " $cbQosObjectsIndex"
				append droplist " $cbQosCMDropPkt_value"
				append droplist " $cbQosCMPrePolicyByte_value"
				append droplist " $cbQosCMPostPolicyByte_value"
				set current_cbQos_logged 1
				if {$verbose == 1} {
				    puts "get_all_drops_for_the_interface: "
				    puts "droplist for an existing entry "
				    puts "interface: $droplist"
				}
			    } else {
				if {$droplist !=""} {
				    append droplist "\n"
				}
				append droplist "$pre_cbQosPolicyIndex"
				append droplist " $pre_cbQosObjectsIndex"
				append droplist " $pre_cbQosCMDropPkt_value"
				append droplist \
					" $pre_cbQosCMPrePolicyByte_value"
				append droplist \
					" $pre_cbQosCMPostPolicyByte_value"
				if {$verbose == 1} {
				    puts "get_all_drops_for_the_interface: "
				    puts "droplist for an non-existing entry "
				    puts "at same policyindex is: $droplist"
				}
			    }
			} else {
			    if {$droplist !=""} {
				append droplist "\n"
			    }
			    append droplist "$pre_cbQosPolicyIndex"
			    append droplist " $pre_cbQosObjectsIndex"
			    append droplist " $pre_cbQosCMDropPkt_value"
			    append droplist " $pre_cbQosCMPrePolicyByte_value"
			    append droplist " $pre_cbQosCMPostPolicyByte_value"
			    
			    if {$verbose == 1} {
				puts "get_all_drops_for_the_interface: "
				puts "droplist for an non-existing entry "
				puts "one both PolicyIndex and ObjectsIndex "
				puts "is $droplist"
			    }
			}
		    } else {
			
			if {$current_cbQos_logged != "1" } {
			    if { $verbose == 1 } {
				puts "get_all_drops_for_the_interface: "
				puts "current_cbQos_logged is "
				puts "$current_cbQos_logged"
			    }
			    if {$droplist !=""} {
				append droplist "\n"
			    }
			    append droplist "$cbQosPolicyIndex"
			    append droplist " $cbQosObjectsIndex"
			    append droplist " $cbQosCMDropPkt_value"
			    append droplist " $cbQosCMPrePolicyByte_value"
			    append droplist " $cbQosCMPostPolicyByte_value"
			}
			set run_again "no"
		    }
		}
		close $drop_fileId
		if { $verbose == 1 } {
		    puts "get_all_drops_for_the_interface: "
		    puts "droplist for all cases is: $droplist"
		}
	    } else {
		if {$verbose == 1} {
		    puts "get_all_drops_for_the_interface: "
		    puts "There is no file here, need to write the value "
		    puts "in $drop_file"
		}
		append droplist "$cbQosPolicyIndex"
		append droplist " $cbQosObjectsIndex"
		append droplist " $cbQosCMDropPkt_value"
		append droplist " $cbQosCMPrePolicyByte_value"
		append droplist " $cbQosCMPostPolicyByte_value"
	    }
	    set drop_fileId [open "$drop_file" w+ 0777]
	    if {$verbose == 1} {
		puts "get_all_drops_for_the_interface: "
		puts "final droplist written to drop_file is $droplist"
	    }
	    puts $drop_fileId "$droplist"
	    close $drop_fileId

	}
   
	if { $cbQosObjectsType_value == 3 } {

            set cbQosMatchStmtName_str_value [get_MatchStmtName \
                    $cbQosConfigIndex_value ]

	    set cbQosMatchPrePolicyPkt_oid "1.3.6.1.4.1.9.9.166.1.16.1.1.2"
	    set cbQosMatchPrePolicyPkt_value [get_query_value_from_oid\
		    $cbQosMatchPrePolicyPkt_oid $cbQosPolicyIndex \
		    $cbQosObjectsIndex]

	    if { $cbQosMatchStmtName_str_value != "" } {
		if { $cbQosMatchPrePolicyPkt_value != "" } {
                    if { $diff_only == "no" } {
                        append msg [get_MatchStmt_output \
                                $cbQosMatchStmtName_str_value\
                                $cbQosMatchPrePolicyPkt_value]
                    }
                }
	    }
	}

	if { $cbQosObjectsType_value == 4 } {

            # If cbQosObjectsType value is 4, it is a queuing
            # Check cbQosQueueingDiscardPkt (1.3.6.1.4.1.9.9.166.1.18.1.1.7)

	    set cbQosQueueingDiscardPkt_oid "1.3.6.1.4.1.9.9.166.1.18.1.1.7"
	    set cbQosQueueingDiscardPkt [get_query_value_from_oid \
		    $cbQosQueueingDiscardPkt_oid $cbQosPolicyIndex \
		    $cbQosObjectsIndex]

	    set queueing_output "Queueing Discard Pkt: \
		    $cbQosQueueingDiscardPkt"
	    if { $cbQosQueueingDiscardPkt != "" } {
                    append msg " $queueing_output"
	    }
	}

	if { $cbQosObjectsType_value == 5 } {
            # If cbQosObjectsType value is 5, it is a ransom detect, 
            # check its query to decide its discard value

	    set cbQosREDRandomDropPkt_oid "1.3.6.1.4.1.9.9.166.1.20.1.1.2"
	    set cbQosREDRandomDropPkt [get_query_value_from_oid \
		    $cbQosREDRandomDropPkt_oid $cbQosPolicyIndex \
		    $cbQosObjectsIndex]

	    set redomdrop_output "Randomency Drop Pkt: \
		    $cbQosREDRandomDropPkt"

	    if { $redomdrop_output != "" } {
                    append msg " $redomdrop_output"
	    }
	}

	if { $cbQosObjectsType_value == 6} {
            # If cbQosObjectsType value is 6, it is a traffic shaping
            # Check its query to decide its discard value

	    set cbQosTSStatsDropByte_oid "1.3.6.1.4.1.9.9.166.1.19.1.1.8"
	    set cbQosTSStatsDropByte [get_query_value_from_oid \
		    $cbQosTSStatsDropByte_oid $cbQosPolicyIndex \
		    $cbQosObjectsIndex]
            if { $verbose == 1 } {
                puts "cbQosTSStatsDropByte = $cbQosTSStatsDropByte"
            }
            if { $cbQosTSStatsDropByte != "" } {
                set traffic_shapping_output "Traffic shapping drop byte: \
		    $cbQosTSStatsDropByte"
                if { $diff_only == "no" } {
                    append msg " $traffic_shapping_output"
                }
	    }
	}

	if { $cbQosObjectsType_value == 7 } {
            # If cbQosObjectsType value is 7, it is a police,
            # check its police tp decide drop counter

	    # cbQosPoliceActionCfgExceed (1.3.6.1.4.1.9.9.166.1.21.1.1.4) is
            # not a supported snmp in eem, so integer oid  needs to be used. 

            set cbQosPoliceActionCfgExceed_oid \
                    "1.3.6.1.4.1.9.9.166.1.21.1.1.4"
	    set cbQosPoliceCfgExceedAction_query [sys_reqinfo_snmp oid \
                $cbQosPoliceActionCfgExceed_oid.$cbQosConfigIndex_value.0 \
                    get_type exact]
	    if {$verbose == 1} {
		puts "get_all_drops_for_the_interface: "
		puts " cbQosPoliceCfgExceedAction_query is \
			$cbQosPoliceCfgExceedAction_query"
	    }

	    set cbQosPoliceCfgExceedAction_value [get_snmp_query_value \
		    $cbQosPoliceCfgExceedAction_query]
	    if {$verbose == 1} {
		puts "get_all_drops_for_the_interface: "
		puts "cbQosPoliceCfgExceedAction_value = \
			$cbQosPoliceCfgExceedAction_value"
	    }

	    if { $cbQosPoliceCfgExceedAction_value == 5 } {

		# If cbQosPoliceCfgExceedAction value is 5, it means police
		# would have an action drop on these packets. Need to
		# check its drop packets and drop bytes.
                # cbQosPliceExceededPkt (1.3.6.1.4.1.9.9.166.1.17.1.1.9)
		# is not a supported snmp eem, needs to use its integer oid
		# to get its query.

		set cbQosPoliceExceededPkt_oid \
                        "1.3.6.1.4.1.9.9.166.1.17.1.1.9"
		set cbQosPoliceExceededPkt \
	        cbQosPoliceExceededPkt_oid.$cbQosPolicyIndex.$cbQosObjectsIndex
		set cbQosPoliceExceededPkt_query [sys_reqinfo_snmp oid \
		    $cbQosPoliceExceededPkt get_type exact]
		if {$verbose == 1} {
		    puts "get_all_drops_for_the_interface: "
		    puts "cbQosPoliceExceededPkt_query is \
			    $cbQosPoliceExceededPkt_query"
		}

		set cbQosPoliceExceededByte_oid \
			"1.3.6.1.4.1.9.9.166.1.17.1.1.12"
		set cbQosPoliceExceededByte [get_query_value_from_oid \
		    $cbQosPoliceExceededByte_oid $cbQosPolicyIndex \
		    $cbQosObjectsIndex]

                set cbQosPoliceCfgExceedAction_value "drop"
	    }
	    
            set police_output "\nPOLICE  <exceeded byte>: \
                    $cbQosPoliceExceededByte  <exceed action>:\
		    $cbQosPoliceCfgExceedAction_value"

	    if { $police_output != "" } {
                if { $diff_only == "no" } {
                    append msg " $police_output"
                }
	    }
	}
 

	if { $cbQosObjectsType_value == 8} {
	    # If cbQosObjectsType value is 8, it means it is a set, we will 
	    # check its query by configIndex value configuration and decide 
	    # its discard value
	    set cbQosSetDiscardClassPkt64_oid "1.3.6.1.4.1.9.9.166.1.24.1.1.8"
	    set cbQosSetDiscardClassPkt64 [get_query_value_from_oid \
		    $cbQosSetDiscardClassPkt64_oid $cbQosPolicyIndex \
		    $cbQosObjectsIndex]

	    set set_output "set Discard Class Packet: \
		    $cbQosSetDiscardClassPkt64"
	    if { $set_output!= "" } {
                if { $diff_only == "no" } {
                    append msg " $set_output"
                }
	    }
	}

	set cbQosObjectsType_query [sys_reqinfo_snmp oid \
		$cbQosObjectsType_oid.$cbQosPolicyIndex.$cbQosObjectsIndex \
		get_type next]
	if {$verbose == 1} {
	    puts "get_all_drops_for_the_interface: "
	    puts "cbQosObjectsType_query is $cbQosObjectsType_query"
	}
	
	set cbQosObjectsType_static_oid [ \
		get_snmp_static_oid_from_2_index_query \
                $cbQosObjectsType_query]

        set old_cbQosPolicyIndex $cbQosPolicyIndex

        set cbQosPolicyIndex [ get_snmp_oid_first_index \
                $cbQosObjectsType_query]

        set cbQosObjectsIndex [get_snmp_oid_last_index \
                $cbQosObjectsType_query]

        if { $cbQosObjectsType_static_oid == \
                "1.3.6.1.4.1.9.9.166.1.5.1.1.3"} {
            if { $old_cbQosPolicyIndex != $cbQosPolicyIndex } {
                if {$verbose == 1} {
                    puts "get_all_drops_for_the_interface: "
                    puts "The ObjectsType is for another policymap"
                }
                break
            }
	    if {$verbose == 1} {
		puts "get_all_drops_for_the_interface: "
		puts "The query is still for objectsType"
	    }
	    set cbQosObjectsType_value [get_snmp_query_value \
		    $cbQosObjectsType_query]
	    if {$verbose == 1} {
		puts "get_all_drops_for_the_interface: "
		puts "cbQosObjectsType_value is $cbQosObjectsType_value"
	    }
	} else {
	    break;
	}
    }

    if {$verbose == 1} {
	puts "get_all_drops_for_the_interface: "
        puts "get all counters for this interface $msg"
    }

    return $msg 

}

##############################################################################
#
# Description: get_mib_data 
#              Get mibdata from eem mib tree for one interface on user
#              specified direction
#
# In: diff_only: If the user wants to have diff_only counters (yes/no)
# Out: class_name: Which part of drop counters the user wants. It can be
#                  policmap, classmap, match statement etc.
#
##############################################################################

proc get_mib_data {diff_only class_name intf dir drop_file} {

    global verbose
    set msg_delimiter "--------------------------------------------------------------"

    if { $verbose == 1 } {
        puts "get_mib_data: diff_only= $diff_only class_name = $class_name "
	puts "intf= $intf dir = $dir drop_file = $drop_file"
    }

    if { $diff_only == "yes" } {
	if { $verbose == 1 } {
            puts "get_mib_data: report diff_only for classmap difference"
	}
	set final_result [get_all_drops_for_the_interface "yes" $intf $dir \
		$drop_file]

        # means diff only, do not get raw counter anymore.
        return $final_result
    }

    if {$class_name != " " } {

	# check if it is a valid class, it has to be a valid 
	# policymap/classmap/match statement police and decide what it is. 

	# Check if this class is a polcymap

	# cbQosPolicyMapName oid ( 1.3.6.1.4.1.9.9.166.1.6.1.1.1 ) is not
	# supported by snmp name, so we use integer oid to for its snmp query.
	set cbQosPolicyMapName_oid "1.3.6.1.4.1.9.9.166.1.6.1.1.1"
	
	set cbQosConfigIndex [find_index_from_one_index_table $class_name \
		$cbQosPolicyMapName_oid]

        if { $cbQosConfigIndex != "" } {
	    if { $verbose == 1 } {
		puts "get_mib_data: Found a match for policymap, check it"
	    }

            # At this point, diff_only can be only no.
            set final_result [get_all_drops_for_the_interface "no" \
                    $intf $dir $drop_file]
	} else {

            if { $verbose == 1 } {
                puts "Class name is not a policymap, check classmap ..."
            }

            set ifindex_no [get_ifindex_no $intf]

            set cbQosPolicyIndex [find_policy_index_from_one_index_table \
                    $ifindex_no $dir]

            if { $verbose == 1 } {
                puts "get_mib_data: cbQosPolicyIndex is $cbQosPolicyIndex"
            }

            if { $cbQosPolicyIndex == "" } {
                append final_result \n\n "The line in inputfile is "
                append final_result "\"$diff_only $class_name $intf $dir\", "
                append final_result "\"$dir\" is not a valid service policy "
                append final_result "direction for interface \"$intf\",  "
                append final_result "Please check your inputfile and rerun "
                append final_result "it again "
                return $final_result
            }

	    # Check if the class is a classmap
	    set cbQosCMNameOid "1.3.6.1.4.1.9.9.166.1.7.1.1.1"

	    set cbQosConfigIndex [find_index_from_one_index_table \
		    $class_name $cbQosCMNameOid]

            if { $cbQosConfigIndex != "" } {

                # Check if the classmap ConfigIndex is in the policy map the
                # interface configured

                if { $verbose == 1 } {
                    puts "get_mib_data: cbQosConfigIndex is $cbQosConfigIndex"
                }

                set cbQosConfigIndex_oid "1.3.6.1.4.1.9.9.166.1.5.1.1.2"
                set cbQosObjectsIndex [find_2nd_index_from_2_index_table \
                        $cbQosConfigIndex $cbQosPolicyIndex \
                        $cbQosConfigIndex_oid]

                if { $cbQosObjectsIndex == "" } {
                    append final_result \n\n "The line in inputfile is "
                    append final_result "\"$diff_only $class_name $intf $dir\", "
                    append final_result "\"$class_name\" is not a valid classmap "
                    append final_result "for interface \"$intf\", Please "
                    append final_result "check your inputfile and rerun it again "
                    return $final_result
                }


                if { $verbose == 1 } {
                    puts "get_mib_data: Found match for classmap, check \
                            counters, cbQosObjectsIndex is $cbQosObjectsIndex "
                }
                    
                set cbQosPolicyMapName [get_PolicyMapName_by_PolicyIndex \
                        $cbQosPolicyIndex]

                set cbQosCMInfo_str_value [get_CMInfo \
                        $cbQosConfigIndex]
                set MatchStmt [get_MatchStmt $cbQosPolicyIndex \
                        $cbQosConfigIndex]

                set classmap_counters [classmap_counter_report \
                        $cbQosPolicyIndex $cbQosObjectsIndex]

                if { $classmap_counters  != 0 } {
                    set cbQosCMDropPkt_value [lindex $classmap_counters 0]
                    set cbQosCMPrePolicyByte_value [lindex \
                            $classmap_counters 1]
                    set cbQosCMPostPolicyByte_value [lindex \
                            $classmap_counters 2]
                }

                append final_result  [get_intf_policy_msg $intf $dir \
                        $cbQosPolicyMapName]
                append final_result [get_CM_output \
                        $class_name $cbQosCMInfo_str_value \
                        $cbQosCMDropPkt_value \
                        $cbQosCMPrePolicyByte_value \
                        $cbQosCMPostPolicyByte_value $MatchStmt]

	    } else { #end of classmap config index
		# Check if it is a matchstatement

                if { $verbose == 1 } {
                    puts "Class name is not a classmap, check matchstament..."
                }

                set ifindex_no [get_ifindex_no $intf]

                set cbQosPolicyIndex [find_policy_index_from_one_index_table \
                        $ifindex_no $dir]

                set cbQosMatchStmtName_oid "1.3.6.1.4.1.9.9.166.1.8.1.1.1"
                set cbQosConfigIndex [find_index_from_one_index_table \
                        $class_name $cbQosMatchStmtName_oid]

                if { $cbQosConfigIndex != "" } {

		    if {$verbose == 1} {
			puts "get_mib_data: Found a match for matchstatement"
		    }

                    set ConfigIndex_for_the_interface $cbQosConfigIndex
                    set cbQosConfigIndexoid "1.3.6.1.4.1.9.9.166.1.5.1.1.2"

                    set cbQosObjectsIndex \
                            [find_2nd_index_from_2_index_table \
                            $ConfigIndex_for_the_interface $cbQosPolicyIndex \
                            $cbQosConfigIndexoid ]

                    if { $cbQosObjectsIndex != "" } {

                        if {$verbose == 1} {
                            puts "get_mib_data: cbQosPolicyIndex is "
                            puts "$cbQosPolicyIndex cbQosObjectsIndex is "
                            puts "$cbQosObjectsIndex"
                        }

                        set cbQosMatchPrePolicyPkt_oid \
                                "1.3.6.1.4.1.9.9.166.1.16.1.1.2"
                        set cbQosMatchPrePolicyPkt_value \
                                [get_query_value_from_oid \
                                $cbQosMatchPrePolicyPkt_oid \
                                $cbQosPolicyIndex \
                                $cbQosObjectsIndex]
                        if { $cbQosMatchPrePolicyPkt_value != "" } {
                            append final_result [get_MatchStmt_output \
                                    $class_name \
                                    $cbQosMatchPrePolicyPkt_value]
                        }
                        
                    }   
		} else {
                    append final_result \n\n "The line in inputfile is "
                    append final_result "\"$diff_only $class_name $intf $dir\", "
                    append final_result "\"$class_name\" is not a valid "
                    append final_result "policymap/classmap/match statement for "
                    append final_result "interface \"$intf\", Please check "
                    append final_result "your inputfile and rerun it again "
                }
	    }
	}
    } 

    return $final_result

}

############################################################################
#
# main function is as the following:
# Read an input file specified from environment variables defined by ios 
# inorder to get parameter for qos drop. 
#  Requires:    - the file "filename" must exist
#               - the input file must have a special format
#
#############################################################################

set inputfile $_qos_file_path
append inputfile $_qos_input_file
 
if {[catch {set fileId [open $inputfile r]} err] == 1} {
    puts "The script needs an input file to run"
    return;
}

set drop_file $_qos_file_path
append drop_file "drop_file"

set run_again "yes"
set final_output ""

while {$run_again == "yes"} {
    gets $fileId value

    if { $value != "" } {
	# trim off all the leading and trailing spaces

	set value [string trim $value]

        # In case some one configure the line as with coma, 
        # for example: "no, PingPackets, gi0/0, output"
        if [regexp {,} $value] {
            regsub -all ","  $value  {} value
        }

	if { $verbose == 1 } {
	    puts "\nMain: using value is: $value"
	}

	set diff_only [lindex $value 0]

	if [regexp {^[\#]} $diff_only] {
	    if { $verbose == 1 } {
		#ignore it
		puts "main: Ignored line: $value"
	    }
	    
	} else {
	    set value_list_length [llength [split $value " "]]
	    if { $value_list_length != 4 } {
		#ignore it
                append final_output \n\n "The line in inputfile is "
                append final_output "\"$value\", which does not have a "
                append final_output "standard format, each line has to have 4 "
                append final_output "arguments, Now it has $value_list_length "
                append final_output "length, Please check your "
                append final_output "inputfile and rerun it again "
	    } else {
		set class_name [lindex $value 1]
		set intf [lindex $value 2]
		set dir [lindex $value 3]
		set output_to_customer [get_mib_data $diff_only $class_name \
			$intf $dir $drop_file]
                if { $output_to_customer != "" } {
                    append final_output " $output_to_customer"
                }
	    }
	}
    } else {
	if {$verbose == 1} {
	    puts "\nMain: This is the end of file"
	}
	set run_again "no"
    }
}

close $fileId

set msg_header "\n\nTHE SERVICE DIAGNOSTIC MESSAGE FOR QOS DROP COUNTER is:" 
set msg $msg_header

if { $final_output != "" } {
    append msg $final_output
} else {
    return
}

if { $msg == $msg_header } {
    append msg \n\n "No drop counter is reported"
}

append msg \n\n "--------------------------------------------------------"
append msg "------"
action_syslog msg $msg

# Only email would attach the log history and event history

if { $send_email == 0 } {
   send_info $config_history $event_history 
} else {
   send_result_by_email $config_history $event_history $msg $_email_from \
       $_email_to $_email_cc $_email_server
}
